@@ -6,46 +6,77 @@ function(DF, plots=TRUE, xyTopLeft=TRUE, ABalt=c("two.sided", "less", "greater")
     #####-----------------------------------------------------------------------
     ## make sure DF has the required variable names and at least two groups
     varNames <- names(DF)                # what variables are present
-    needs    <- c("Series", "Distance", "Aim.X", "Aim.Y", "Point.X", "Point.Y")  # required
-    has      <- needs %in% varNames      # what we have of the required ones
-    if(!all(has)) {
-        stop(cat("the data frame is missing variable(s)\n", needs[!has], "\n"))
+    needsSer <- "Series"                 # required
+    needsXY1 <- c("Point.X", "Point.Y")  # coordinates have have this name
+    needsXY2 <- c("X", "Y")              # or this
+    wantsDst <- "Distance"               # useful
+    wantsAIM <- c("Aim.X", "Aim.Y")      # useful
+    hasSer   <- needsSer %in% varNames   # required ones we have
+    hasXY1   <- needsXY1 %in% varNames
+    hasXY2   <- needsXY2 %in% toupper(varNames)
+    hasDst   <- wantsDst %in% varNames   # useful ones we have
+    hasAIM   <- wantsAIM %in% varNames   # useful ones we have
+ 
+    if(!all(hasSer)) {
+        stop(c("The data frame is missing variable\n",
+               paste(needsSer[!hasSer], collapse=" "), "\n"))
     }
 
+    if(!xor(all(hasXY1), all(hasXY2))) {
+        stop("Coordinates must be named either X, Y or Point.X, Point.Y\n")
+    }
+
+    if(!all(hasDst)) {
+        warning(c("\nThe data frame is missing variable\n",
+                  paste(wantsDst[!hasDst], collapse=" "), "\n",
+                  "Distance is assumed to be 100\n"))
+        DF$Distance <- 100
+    }
+
+    if(!all(hasAIM)) {
+        warning(c("\nThe data frame is missing variable(s)\n",
+            paste(wantsAIM[!hasAIM], collapse=" "), "\n",
+            "Point of Aim is assumed to be in (0,0)\n"))
+        DF$Aim.X <- 0
+        DF$Aim.Y <- 0
+    }
+    
     #####-----------------------------------------------------------------------
     ## prepare data
     res <- vector("list", 0)             # empty list to later collect the results
-    DF$Series <- droplevels(DF$Series)   # remove all non-used factor levels
+    if(!is.factor(DF$Series)) {          # make sure Series is a factor
+        DF$Series <- as.factor(DF$Series)
+    } else {
+        DF$Series <- droplevels(DF$Series)   # remove all non-used factor levels
+    }
 
     ## check if we have enough groups and points per group
-    if(nlevels(DF$Series) < 2)    { stop("we need >= 2 groups for a comparison") }
-    if(any(table(DF$Series) < 2)) { stop("we need >= 2 points in each group") }
+    if(nlevels(DF$Series) < 2) { stop("We need >= 2 groups for a comparison") }
+    if(any(xtabs(~ Series, data=DF) < 2)) { stop("We need >= 2 points in each group") }
 
-    ## prepare data: coords relative to point of aim
-    ## y-coords exported from OnTarget: (0,0) is top-left
-    DF$X <- DF$Point.X - DF$Aim.X        # x-coords
-    if(xyTopLeft) {
-        DF$Y <- -(DF$Point.Y - DF$Aim.Y)
-    } else {
-        DF$Y <-   DF$Point.Y - DF$Aim.Y
-    }
-    xy        <- cbind(DF$X, DF$Y)       # new (x,y)-coords as matrix
+    ## prepare data: get (x,y)-coords relative to point of aim as matrix
+    xy <- getXYmat(DF, xyTopLeft=xyTopLeft)
+    DF <- cbind(DF, xy)
     dstTarget <- tapply(DF$Distance, DF$Series, mean)  # distances to target
 
     ## for each group extract the new (x,y)-coords as a matrix
-    xyL <- lapply(split(DF, DF$Series), function(x) {
-                  data.matrix(subset(x, select=c("X", "Y"))) } )
-    nS <- length(xyL)                    # total number of series
+    extractXY <- function(x) {
+        DFsub <- x[ , which(!(names(x) %in% c("X", "Y")))]
+        getXYmat(DFsub, xyTopLeft=xyTopLeft)
+    }
+    xyL  <- lapply(split(DF, DF$Series), extractXY)
+    nS   <- length(xyL)                  # total number of series
+    nObs <- sapply(xyL, length)          # number of obs per series
     names(xyL) <- paste("Series", levels(DF$Series), sep="")
 
     #####-----------------------------------------------------------------------
     ## location measures
-    res$ctr     <- sapply(xyL, colMeans)  # group centers
+    res$ctr     <- sapply(xyL, colMeans)     # group centers
     distPOA     <- sqrt(colSums(res$ctr^2))  # distances to point of aim
-    distPOAmoa  <- getMOA(distPOA, dstTarget, conversion)
+    distPOAmoa  <- getMOA(distPOA, dst=dstTarget, conversion=conversion)
     res$distPOA <- rbind(unit=distPOA, MOA=distPOAmoa)
 
-    ## multivariate location test for equal group centers
+    ## multivariate location test for equal group centers (relative to POA)
     res$MANOVA <- anova(lm(cbind(X, Y) ~ Series, data=DF), test="Wilks")
 
     #####-----------------------------------------------------------------------
@@ -55,51 +86,70 @@ function(DF, plots=TRUE, xyTopLeft=TRUE, ABalt=c("two.sided", "less", "greater")
 
     #####-----------------------------------------------------------------------
     ## spread measures
-    ## standard deviations for x- and y-coords
-    covXY    <- lapply(xyL, cov)         # covariance matrices (x,y)-coords
-    res$sdXY <- sapply(covXY, function(x) sqrt(diag(x)))  # std devs
+    ## standard deviations for x- and y-coords with parametric CIs
+    alpha <- 0.05
+    getSDxyCI <- function(x) {
+        N    <- nrow(x)
+        sdXY <- sqrt(diag(cov(x)))       # standard deviations
+
+        ## and their parametric 95%-confidence intervals
+        sdXci  <- sqrt((N-1)*sdXY["X"]^2 / qchisq(c(1-(alpha/2), alpha/2), N-1))
+        sdYci  <- sqrt((N-1)*sdXY["Y"]^2 / qchisq(c(1-(alpha/2), alpha/2), N-1))
+        sdXYci <- c(sdXci, sdYci)
+
+        names(sdXYci) <- c("sdX (", "sdX )", "sdY (", "sdY )")
+        return(sdXYci)
+    }
+
+    ## sd and sd CIs as separate lists for unit of measurement and MOA
+    sdXYunit   <- lapply(xyL, function(x) sqrt(diag(cov(x)))) # standard deviations
+    sdXYmoa    <- Map(getMOA, sdXYunit,   dst=dstTarget, conversion=conversion)
+    sdXYciUnit <- lapply(xyL, getSDxyCI)             # confidence intervals
+    sdXYciMOA  <- Map(getMOA, sdXYciUnit, dst=dstTarget, conversion=conversion)
+
+    ## add to results list as matrices
+    res$sdXY      <- do.call("cbind", sdXYunit)
+    res$sdXYmoa   <- do.call("cbind", sdXYmoa)
+    res$sdXYci    <- do.call("cbind", sdXYciUnit)
+    res$sdXYciMOA <- do.call("cbind", sdXYciMOA)
 
     ## mean distances to group center
-    dstList   <- lapply(xyL, getDistToCtr) # distances to group center
-    dstCtr    <- do.call("c", dstList)     # as vector
-    dstCtrMOA <- getMOA(dstCtr, DF$Distance, conversion)     # as MOA
-    meanDistToCtr     <- tapply(dstCtr,    DF$Series, mean)  # mean distance
-    meanDistToCtrMOA  <- tapply(dstCtrMOA, DF$Series, mean)  # as MOA
-    res$meanDistToCtr <- rbind(unit=meanDistToCtr, MOA=meanDistToCtrMOA)
+    meanDstCtr    <- sapply(xyL, function(x) mean(getDistToCtr(x)))
+    meanDstCtrMOA <- Map(getMOA, meanDstCtr, dst=dstTarget, conversion=conversion) # MOA
+    res$meanDistToCtr <- rbind(unit=meanDstCtr, MOA=meanDstCtrMOA)
     colnames(res$meanDistToCtr) <- names(xyL)
 
     ## maximum pairwise distance = maximum group spread
     maxPD        <- lapply(xyL, getMaxPairDist)  # max pairwise distance
     maxSpread    <- sapply(maxPD, function(x) { x$d } )
     maxPDidx     <- sapply(maxPD, function(x) { x$idx } )
-    maxSpreadMOA <- getMOA(maxSpread, dstTarget, conversion)  # as MOA
+    maxSpreadMOA <- getMOA(maxSpread, dst=dstTarget, conversion=conversion)
     res$maxPairDist <- rbind(unit=maxSpread, MOA=maxSpreadMOA)
 
-    ## bounding box width and height
-    ## bbs             <- lapply(xyL, getBoundingBox)   # bounding boxes
-    bbs             <- lapply(xyL, getMinBBox)
-    groupWidth      <- sapply(bbs, function(x) { x$width } )
-    groupHeight     <- sapply(bbs, function(x) { x$height } )
-    groupWidthMOA   <- getMOA(groupWidth,  dstTarget, conversion)
-    groupHeightMOA  <- getMOA(groupHeight, dstTarget, conversion)
-    res$groupWidth  <- rbind(unit=groupWidth,  MOA=groupWidthMOA)
-    res$groupHeight <- rbind(unit=groupHeight, MOA=groupHeightMOA)
+    ## bounding box figure of merit and diagonal
+    ## bbs         <- lapply(xyL, getBoundingBox)   # bounding boxes
+    bbs        <- lapply(xyL, getMinBBox)
+    bbFoM      <- sapply(bbs, function(x) { x$FoM } )
+    bbDiag     <- sapply(bbs, function(x) { x$diag } )
+    bbFoMMOA   <- getMOA(bbFoM,  dst=dstTarget, conversion=conversion)
+    bbDiagMOA  <- getMOA(bbDiag, dst=dstTarget, conversion=conversion)
+    res$bbFoM  <- rbind(unit=bbFoM,  MOA=bbFoMMOA)
+    res$bbDiag <- rbind(unit=bbDiag, MOA=bbDiagMOA)
 
     ## radius of minimum enclosing circle
     minCircs         <- lapply(xyL, getMinCircle)
     minCircleRad     <- sapply(minCircs, function(x) { x$rad } )     # radius
-    minCircleRadMOA  <- getMOA(minCircleRad, dstTarget, conversion)  # as MOA
+    minCircleRadMOA  <- getMOA(minCircleRad,
+                               dst=dstTarget, conversion=conversion) # as MOA
     res$minCircleRad <- rbind(unit=minCircleRad, MOA=minCircleRadMOA)
 
     ## 50% circular error probable
-    CEPlist <- vector("list", nS)
-    for(i in seq(along=numeric(nS))) {
-        CEPlist[[i]] <- getCEP(xyL[[i]], dstTarget[i], conversion)
-    }
-    CEPrandL <- lapply(CEPlist, function(x) { x$RAND[ , "50%", drop=FALSE] } )
-    CEPmat   <- do.call("cbind", CEPrandL)    # as matrix
+    CEPlist <- Map(getCEP, xyL, dstTarget=dstTarget, conversion=conversion,
+                   type="Rayleigh", accuracy=FALSE)
+    CEPrayL <- lapply(CEPlist, function(x) { x$Rayleigh[ , "50%", drop=FALSE] } )
+    CEPmat  <- do.call("cbind", CEPrayL)     # as matrix
     colnames(CEPmat) <- names(xyL)
-    res$CEPrand <- CEPmat
+    res$CEPray <- CEPmat
 
     #####-----------------------------------------------------------------------
     ## tests for equal spread
@@ -107,20 +157,28 @@ function(DF, plots=TRUE, xyTopLeft=TRUE, ABalt=c("two.sided", "less", "greater")
     ##             Kruskal-Wallis for distance to center
     ## > 2 groups: Fligner-Killeen for x- and y-coords
     ##             Wilcoxon Rank Sum (= Mann-Whitney U) for distance to center
+    dstCtrL   <- lapply(xyL, getDistToCtr) # distances to group center
+    dstCtrGrp <- unlist(dstCtrL)           # grouped by Series
+    names(dstCtrGrp) <- NULL
+
+    ## create data frame with added Series factor
+    dstCtrDf  <- data.frame(dstCtr=dstCtrGrp,
+                            Series=factor(rep(1:nlevels(DF$Series), nObs), labels=levels(DF$Series)))
+
     if(nS == 2) {                        # compare two groups
-        res$AnsariX  <- coin::ansari_test(     X ~ Series, alternative=ABalt,
+        res$AnsariX  <- coin::ansari_test(X ~ Series, alternative=ABalt,
                                           data=DF, distribution="exact")
-        res$AnsariY  <- coin::ansari_test(     Y ~ Series, alternative=ABalt,
+        res$AnsariY  <- coin::ansari_test(Y ~ Series, alternative=ABalt,
                                           data=DF, distribution="exact")
         res$Wilcoxon <- coin::wilcox_test(dstCtr ~ Series, alternative=Walt,
-                                          data=DF, distribution="exact")
+                                          data=dstCtrDf, distribution="exact")
     } else {                             # compare more than two groups
         res$FlignerX <- coin::fligner_test(X ~ Series, data=DF,
                                            distribution="approximate")  # x
         res$FlignerY <- coin::fligner_test(Y ~ Series, data=DF,
                                            distribution="approximate")  # y
         res$Kruskal  <- coin::kruskal_test(dstCtr ~ Series,    # dist to center
-                                           data=DF, distribution="approximate")
+                                           data=dstCtrDf, distribution="approximate")
     }
 
     if(plots) {
@@ -130,19 +188,23 @@ function(DF, plots=TRUE, xyTopLeft=TRUE, ABalt=c("two.sided", "less", "greater")
         syms <- c(4, 16, 2, 1, 6, 8, 3, 5, 7, 9:13, 15, 17:25)  # data symbols
         cols <- getColors(nS)            # colors
         
-        if(nS > length(syms)) { stop(paste("at most", length(syms), "series possible")) }
+        if(nS > length(syms)) {
+            stop(paste("At most", length(syms), "series possible"))
+        }
         
         devNew()                         # open new diagram
-        plot(DF$X, DF$Y, asp=1, lwd=2, xlab="X", ylab="Y",
+        plot(Y ~ X, data=DF, asp=1, lwd=2, xlab="X", ylab="Y",
              pch=syms[unclass(DF$Series)], col=cols[unclass(DF$Series)],
              main="Groups with error ellipses")
         abline(v=0, h=0, col="lightgray")  # add point of aim
         
         ## add confidence ellipses and group centers
-        for(i in seq(along=numeric(nS))) {
+        covXY <- lapply(xyL, cov)
+        for(i in seq(along=xyL)) {
             drawEllipse(res$ctr[ , i], covXY[[i]], radius=1, fg=cols[i],
                         lwd=2, pch=syms[i], cex=3)
-            points(res$ctr[1, i], res$ctr[2, i], pch=syms[i], col=cols[i], cex=3, lwd=2)
+            points(res$ctr[1, i], res$ctr[2, i], pch=syms[i], col=cols[i],
+                   cex=3, lwd=2)
         }
         
         ## add legend
@@ -158,17 +220,19 @@ function(DF, plots=TRUE, xyTopLeft=TRUE, ABalt=c("two.sided", "less", "greater")
                        minCircs[[1]]$ctr[2] + c(-minCircs[[1]]$rad, minCircs[[1]]$rad)))
 
         devNew()                         # open new diagram
-        plot(DF$X, DF$Y, asp=1, xlim=xLims, ylim=yLims, xlab="X", ylab="Y", lwd=2,
+        plot(Y ~ X, data=DF, asp=1, xlim=xLims, ylim=yLims,
+             xlab="X", ylab="Y", lwd=2,
              pch=syms[unclass(DF$Series)], col=cols[unclass(DF$Series)],
              main="Groups w/ minimum bounding box & maximum spread")
         abline(v=0, h=0, col="lightgray")  # add point of aim
-        points(res$ctr[1, ], res$ctr[2, ], col=cols[1:nS], pch=syms[1:nS], lwd=2, cex=3)
+        points(res$ctr[1, ], res$ctr[2, ], col=cols[1:nS],
+               pch=syms[1:nS], lwd=2, cex=3)
         
         ## add bounding box and maximum group spread
-        for(i in seq(along=numeric(nS))) {
+        for(i in seq(along=xyL)) {
             bb <- bbs[[i]]
-            ## drawBox(bb$pts[1], bb$pts[2], bb$pts[3], bb$pts[4], fg=cols[i])
-            drawBox2(bb$pts, fg=cols[i])
+            ## drawBox(bb, fg=cols[i])
+            drawBox2(bb, fg=cols[i])
             segments(x0=xyL[[i]][maxPDidx[1, i], 1], y0=xyL[[i]][maxPDidx[1, i], 2],
                      x1=xyL[[i]][maxPDidx[2, i], 1], y1=xyL[[i]][maxPDidx[2, i], 2],
                      col=cols[i])
@@ -181,17 +245,18 @@ function(DF, plots=TRUE, xyTopLeft=TRUE, ABalt=c("two.sided", "less", "greater")
         #####-----------------------------------------------------------------------
         ## diagram: 2D-scatter plot for the (x,y)-distribution
         devNew()                         # open new diagram
-        plot(DF$X, DF$Y, asp=1, xlim=xLims, ylim=yLims, xlab="X", ylab="Y", lwd=2,
+        plot(Y ~ X, data=DF, asp=1, xlim=xLims, ylim=yLims,
+             xlab="X", ylab="Y", lwd=2,
              pch=syms[unclass(DF$Series)], col=cols[unclass(DF$Series)],
              main="Groups w/ minimum enclosing circle and mean dist to center")
         abline(v=0, h=0, col="lightgray")  # add point of aim
-        points(res$ctr[1, ], res$ctr[2, ], col=cols[1:nS], pch=syms[1:nS], lwd=2, cex=3)
+        points(res$ctr[1, ], res$ctr[2, ], col=cols[1:nS], pch=syms[1:nS],
+               lwd=2, cex=3)
         
         ## add circle with mean distance to center and minimum enclosing circle
-        for(i in seq(along=numeric(nS))) {
-            drawCircle(res$ctr[1, i], res$ctr[2, i], radius=meanDistToCtr[i], fg=cols[i])
-            mc <- minCircs[[i]]
-            drawCircle(mc$ctr[1], mc$ctr[2], radius=mc$rad, fg=cols[i])
+        for(i in seq(along=xyL)) {
+            drawCircle(res$ctr[ , i], radius=meanDstCtr[i], fg=cols[i])
+            drawCircle(minCircs[[i]], fg=cols[i])
         }
         
         ## add legend
